import {
  Emitter
} from "./chunk-F6OVXD4L.js";
import {
  uname
} from "./chunk-ULLOUKBT.js";
import {
  DefaultLogger
} from "./chunk-6KWR5DLV.js";
import {
  LogLevelOff
} from "./chunk-E6MPGGAE.js";

// src/common/exec/queue.ts
var SerialQueue = class extends Emitter {
  constructor(opt = {}) {
    super();
    this.queue = [];
    this.waitToFinish = [];
    this.countMax = 0;
    this.countResolved = 0;
    this.paused = false;
    const { name = uname("queue"), logLevel } = opt;
    this.name = name;
    this.log = DefaultLogger(`zeed:queue:${name}`, logLevel ?? LogLevelOff);
  }
  async performNext() {
    this.log("performNext, queue.length =", this.queue.length);
    if (this.currentTask != null) {
      this.log("performNext => skip while another task is running");
      return;
    }
    if (this.paused) {
      this.log("performNext => skip while is paused");
      return;
    }
    while (this.currentTask == null && !this.paused) {
      const info = this.queue.shift();
      this.log(`performNext => ${info?.name}`);
      if (info == null)
        break;
      if (this.countResolved === 0)
        void this.emit("didStart", this.countMax);
      const { name, task, resolve } = info;
      this.currentTask = task();
      let result;
      try {
        this.log.info(`start task ${name}`);
        result = await this.currentTask;
        this.log(`finished task ${name} with result =`, result);
      } catch (err) {
        this.log.warn("Error performing task", err);
      }
      resolve(result);
      this.currentTask = void 0;
      this.countResolved += 1;
      void this.emit("didUpdate", this.countMax, this.countResolved);
    }
    if (this.queue.length === 0) {
      void this.emit("didFinish");
      this.countMax = 0;
      this.countResolved = 0;
    }
    while (this.waitToFinish.length > 0)
      this.waitToFinish.shift()();
  }
  /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
  async enqueue(task, opt = {}) {
    const { immediate = false, name = uname(this.name) } = opt;
    if (immediate) {
      this.log.info(`immediate execution ${name}`);
      return await task();
    }
    this.log(`enqueue ${name}`);
    return new Promise((resolve) => {
      this.queue.push({
        name,
        task,
        resolve
      });
      this.countMax += 1;
      void this.emit("didUpdate", this.countMax, this.countResolved);
      void this.performNext();
    });
  }
  /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
  async enqueueReentrant(task, opt = {}) {
    return this.enqueue(task, {
      immediate: this.currentTask != null,
      name: opt.name
    });
  }
  /** Remove all tasks from queue that are not yet executing. */
  async cancelAll(_unblock = true) {
    this.log("cancelAll");
    void this.emit("didCancel");
    const resolver = this.queue.map((task) => task.resolve);
    this.queue = [];
    resolver.forEach((r) => r(void 0));
    await this.wait();
  }
  /** Pause execution after current task is finished. */
  async pause() {
    this.log("pause");
    this.paused = true;
    await this.wait();
  }
  /** Resume paused queue. */
  resume() {
    this.log("resume");
    this.paused = false;
    void this.performNext();
  }
  /** Wait for all tasks to finish */
  async wait() {
    this.log("wait");
    if (this.currentTask == null && (this.queue.length === 0 || this.paused))
      return;
    return new Promise((resolve) => {
      this.waitToFinish.push(resolve);
    });
  }
  get isPaused() {
    return this.paused;
  }
  get hasTasks() {
    return this.queue.length !== 0;
  }
};

export {
  SerialQueue
};
//# sourceMappingURL=chunk-ENHLOA77.js.map