import { DisposerFunction, Disposer } from './dispose-types.js';
import { UseDisposeConfig } from './dispose-defer.js';
import { LoggerInterface } from './log/log-base.js';

type TimerExecFunction = () => void | Promise<void>;
/**
 * Executes a function after a specified timeout and returns a disposer function
 * that can be used to cancel the timeout.
 *
 * @param fn - The function to execute after the timeout.
 * @param timeout - The timeout duration in milliseconds (default: 0).
 * @returns A disposer function that can be used to cancel the timeout.
 */
declare function useTimeout(fn: TimerExecFunction, timeout?: number): DisposerFunction;
/**
 * Executes a function repeatedly at a specified interval and returns a disposer function
 * that can be used to stop the execution.
 *
 * @param fn - The function to be executed at the specified interval.
 * @param interval - The interval (in milliseconds) at which the function should be executed.
 * @returns A disposer function that can be used to stop the execution of the function.
 */
declare function useInterval(fn: TimerExecFunction, interval: number): DisposerFunction;
/** The interval starts only, when the function is finished. */
declare function useIntervalPause(fn: TimerExecFunction, interval: number, immediately?: boolean): DisposerFunction;
declare function useEventListener(emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]): DisposerFunction;
declare function useEventListenerOnce(emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]): DisposerFunction;
/** Like useDispose but with shorthands for emitter and timers */
declare function useDisposeWithUtils(config?: string | UseDisposeConfig | LoggerInterface): ((strictSync?: boolean) => void | Promise<any>) & {
    disposed: number;
    add: (obj?: Disposer | undefined) => DisposerFunction | undefined;
    remove: (disposable: Disposer) => void | Promise<void>;
    track: (obj?: Disposer | undefined) => DisposerFunction | undefined;
    untrack: (disposable: Disposer) => void | Promise<void>;
    dispose: (strictSync?: boolean) => void | Promise<any>;
    disposeSync: () => void;
    sync: () => void;
    exec: (strictSync?: boolean) => void | Promise<any>;
    getSize(): number;
    isDisposed(): boolean;
} & {
    timeout: (fn: TimerExecFunction, timeout?: number) => DisposerFunction | undefined;
    interval: (fn: TimerExecFunction, interval?: number) => DisposerFunction | undefined;
    intervalPause: (fn: TimerExecFunction, interval?: number) => DisposerFunction | undefined;
    on: (emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]) => DisposerFunction | undefined;
    once: (emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]) => DisposerFunction | undefined;
};
type UseDisposeWithUtils = ReturnType<typeof useDisposeWithUtils>;

export { type TimerExecFunction, type UseDisposeWithUtils, useDisposeWithUtils, useEventListener, useEventListenerOnce, useInterval, useIntervalPause, useTimeout };
